@@ -19,12 +19,13 @@
 
 #This file contains all global objects.
 
-import clock as sing_clock
+version = "0.30"
+
 import ConfigParser
 import pygame
 import os
 import os.path
-import pickle
+import cPickle
 import random
 import sys
 
@@ -32,14 +33,11 @@ import sys
 # are made where needed.
 import locale
 
-import player, base, buttons, tech, item, event, location, buyable, statistics
-from buttons import always, void, exit
+import player, base, tech, item, event, location, buyable, statistics
+import graphics.g
 
 stats = statistics.Statistics()
 
-#screen is the actual pygame display.
-global screen
-
 # Useful constants.
 hours_per_day = 24
 minutes_per_hour = 60
@@ -48,11 +46,6 @@ seconds_per_minute = 60
 seconds_per_hour = 60 * 60
 seconds_per_day = 24 * 60 * 60
 
-#size of the screen. This can be set via command-line option.
-screen_size = (800, 600)
-
-clock = sing_clock.Clock()
-
 #Allows access to the cheat menu.
 cheater = 0
 
@@ -60,8 +53,8 @@ cheater = 0
 # but is untested.
 nosound = 0
 
-#Fullscreen
-fullscreen = 0
+# Enables day/night display.
+daynight = True
 
 #Gives debug info at various points.
 debug = 0
@@ -72,15 +65,17 @@ force_single_dir = False
 #Used to determine which data files to load.
 language = "en_US"
 
-# Try a few locale settings.  First the selected language, then the user's 
-# default, then en_US.  Each one is tried with encoding UTF-8 first, then
-# the default encoding.
+#Makes the intro be shown on the first GUI tick.
+intro_shown = True
+
+# Try a few locale settings.  First the selected language, then the user's
+# default, then en_US.  The selected lanugage and en_US are tried with encoding
+# UTF-8 first, then the default encoding.  The user's default encoding is not
+# paired with UTF-8.
 #
 # If all of that fails, we hope locale magically does the right thing.
 def set_locale():
-    for attempt in [language + ".UTF-8", language, 
-                    locale.getdefaultlocale()[0] + ".UTF-8", "", "en_US.UTF-8",
-                    "en_US"]:
+    for attempt in [language + ".UTF-8", language, "", "en_US.UTF-8", "en_US"]:
         try:
             locale.setlocale(locale.LC_ALL, attempt)
             break
@@ -102,47 +97,10 @@ data_loc = "../data/"
 def quit_game():
     sys.exit()
 
-#colors:
-colors = {}
-
-def fill_colors():
-    colors["white"] = (255, 255, 255, 255)
-    colors["black"] = (0, 0, 0, 255)
-    colors["red"] = (255, 0, 0, 255)
-    colors["green"] = (0, 255, 0, 255)
-    colors["blue"] = (0, 0, 255, 255)
-    colors["dark_red"] = (125, 0, 0, 255)
-    colors["dark_green"] = (0, 125, 0, 255)
-    colors["dark_blue"] = (0, 0, 125, 255)
-    colors["light_red"] = (255, 50, 50, 255)
-    colors["light_green"] = (50, 255, 50, 255)
-    colors["light_blue"] = (50, 50, 255, 255)
-
 strings = {}
+buttons = {}
 help_strings = {}
 
-images = {}
-def load_images():
-    """
-load_images() loads all of the images in the data/images/ directory.
-"""
-    global images
-
-    image_dir = os.path.join(data_loc, "images")
-    image_list = os.listdir(image_dir)
-    for image_filename in image_list:
-
-        # We only want JPGs and PNGs.
-        if len(image_filename) > 4 and (image_filename[-4:] == ".png" or
-         image_filename[-4:] == ".jpg"):
-
-            # We need to convert the image to a Pygame image surface and
-            # set the proper color key for the game.
-            images[image_filename] = pygame.image.load(
-             os.path.join(image_dir, image_filename)).convert()
-            images[image_filename].set_colorkey((255, 0, 255, 255),
-             pygame.RLEACCEL)
-
 sounds = {}
 def load_sounds():
     """
@@ -216,7 +174,7 @@ play_sound() plays a sound from a particular class.
         sys.stderr.write("D: Playing sound %s.\n" % random_sound["filename"])
     random_sound["sound"].play()
 
-delay_time = 0
+delay_time = 1
 music_dict = {}
 
 def load_music():
@@ -289,270 +247,10 @@ def play_music(musicdir="music"):
         pygame.mixer.music.load(random.choice(music_dict[musicdir]))
         pygame.mixer.music.play()
 
-#
-# Font functions.
-#
-
-#Normal and Acknowledge fonts.
-font = []
-font.append([0] * 51)
-font.append([0] * 51)
-
-#given a surface, string, font, char to underline (int; -1 to len(string)),
-#xy coord, and color, print the string to the surface.
-#Align (0=left, 1=Center, 2=Right) changes the alignment of the text
-def print_string(surface, string_to_print, font, underline_char, xy, color, align=0):
-    if align != 0:
-        size = font.size(string_to_print)
-        if align == 1: xy = (xy[0] - size[0]/2, xy[1])
-        elif align == 2: xy = (xy[0] - size[0], xy[1])
-    if underline_char == -1 or underline_char >= len(string_to_print):
-        text = font.render(string_to_print, 1, color)
-        surface.blit(text, xy)
-    else:
-        text = font.render(string_to_print[:underline_char], 1, color)
-        surface.blit(text, xy)
-        size = font.size(string_to_print[:underline_char])
-        xy = (xy[0] + size[0], xy[1])
-        font.set_underline(1)
-        text = font.render(string_to_print[underline_char], 1, color)
-        surface.blit(text, xy)
-        font.set_underline(0)
-        size = font.size(string_to_print[underline_char])
-        xy = (xy[0] + size[0], xy[1])
-        text = font.render(string_to_print[underline_char+1:], 1, color)
-        surface.blit(text, xy)
-
-#Used to display descriptions and such. Automatically wraps the text to fit
-#within a certain width.
-def print_multiline(surface, string_to_print, font, width, xy, color):
-    start_xy = xy
-    string_array = string_to_print.replace("\n", " \\n ").split(" ")
-
-    for string in string_array:
-        string += " "
-        size = font.size(string)
-
-        if string == "\\n ":
-            xy = (start_xy[0], xy[1]+size[1])
-            continue
-        text = font.render(string, 1, color)
-
-        if (xy[0]-start_xy[0])+size[0] > width:
-            xy = (start_xy[0], xy[1]+size[1])
-        surface.blit(text, xy)
-        xy = (xy[0]+size[0], xy[1])
-
-#create dialog with OK button.
-def create_dialog(string_to_print, box_font = None, xy = None, size = (250,250),
-                  bg_color = None, out_color = None, text_color = None):
-    # Defaults that reference other variables, which may not be initialized when
-    # the function is defined.
-    if box_font == None:
-        box_font = font[0][18]
-    if xy == None:
-        xy = ( (screen_size[0] / 2) - 100, 50)
-    if bg_color == None:
-        bg_color = colors["dark_blue"]
-    if out_color == None:
-        out_color = colors["white"]
-    if text_color == None:
-        text_color = colors["white"]
-
-    screen.fill(out_color, (xy[0], xy[1], size[0], size[1]))
-    screen.fill(bg_color, (xy[0]+1, xy[1]+1, size[0]-2, size[1]-2))
-    print_multiline(screen, string_to_print, box_font, size[0]-10,
-            (xy[0]+5, xy[1]+5), text_color)
-    menu_buttons = {}
-    menu_buttons[buttons.make_norm_button((xy[0]+size[0]/2-50,
-            xy[1]+size[1]+5), (100, 50), "OK", "O", font[1][30])] = always(True)
-
-    buttons.show_buttons(menu_buttons)
-
-#create dialog with YES/NO buttons.
-def create_yesno(string_to_print, box_font, xy, size, bg_color, out_color,
-            text_color, button_names=("YES", "NO"), reverse_key_context = False):
-    screen.fill(out_color, (xy[0], xy[1], size[0], size[1]))
-    screen.fill(bg_color, (xy[0]+1, xy[1]+1, size[0]-2, size[1]-2))
-    print_multiline(screen, string_to_print, box_font, size[0]-10,
-                (xy[0]+5, xy[1]+5), text_color)
-    menu_buttons = {}
-    if button_names == ("YES", "NO"):
-        menu_buttons[buttons.make_norm_button((xy[0]+size[0]/2-110,
-                xy[1]+size[1]+5), (100, 50), button_names[0], "Y", font[1][30])] = always(True)
-        menu_buttons[buttons.make_norm_button((xy[0]+size[0]/2+10,
-                xy[1]+size[1]+5), (100, 50), button_names[1], "N", font[1][30])] = always(False)
-    else:
-        menu_buttons[buttons.make_norm_button((xy[0],
-                xy[1]+size[1]+5), -1, button_names[0], button_names[0][0], font[1][30])] = always(True)
-        menu_buttons[buttons.make_norm_button((xy[0]+size[0]-100,
-                xy[1]+size[1]+5), -1, button_names[1], button_names[1][0], font[1][30])] = always(False)
-
-
-    default = False
-    if reverse_key_context:
-        default = True
-
-    return buttons.show_buttons(menu_buttons, 
-                               key_callback=buttons.simple_key_handler(default))
-
-valid_input_characters = ('a','b','c','d','e','f','g','h','i','j','k','l','m',
-                        'n','o','p','q','r','s','t','u','v','w','x','y','z',
-                        'A','B','C','D','E','F','G','H','I','J','K','L','M',
-                        'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
-                        '0','1','2','3','4','5','6','7','8','9','.',' ')
-
-def create_textbox(descript_text, starting_text, box_font, xy, size,
-            max_length, bg_color, out_color, text_color, text_bg_color):
-    screen.fill(out_color, (xy[0], xy[1], size[0], size[1]))
-    screen.fill(bg_color, (xy[0]+1, xy[1]+1, size[0]-2, size[1]-2))
-    screen.fill(out_color, (xy[0]+5, xy[1]+size[1]-30, size[0]-10, 25))
-    print_multiline(screen, descript_text, box_font,
-                                size[0]-10, (xy[0]+5, xy[1]+5), text_color)
-
-    # Cursor starts at the end.
-    global cursor_loc, work_string
-    cursor_loc = len(starting_text)
-
-    def give_text():
-        return work_string
-
-    menu_buttons = {}
-    menu_buttons[buttons.make_norm_button((xy[0]+size[0]/2-50,
-            xy[1]+size[1]+5), (100, 50), "OK", "", font[1][30])] = give_text
-
-    work_string = starting_text
-    sel_button = -1
-
-    key_down_dict = {
-        pygame.K_BACKSPACE: False,
-        pygame.K_DELETE: False,
-        pygame.K_LEFT: False,
-        pygame.K_RIGHT: False
-    }
-    key_down_time_dict = {
-        pygame.K_BACKSPACE: 0,
-        pygame.K_DELETE: 0,
-        pygame.K_LEFT: 0,
-        pygame.K_RIGHT: 0
-    }
-    repeat_timing_dict = {
-        pygame.K_BACKSPACE: 6,
-        pygame.K_DELETE: 6,
-        pygame.K_LEFT: 6,
-        pygame.K_RIGHT: 6
-    }
-
-    def on_tick(tick_len):
-        global cursor_loc, work_string
-        need_redraw = False
-
-        keys = (pygame.K_BACKSPACE, pygame.K_DELETE, pygame.K_LEFT, 
-                pygame.K_RIGHT)
-        backspace, delete, left, right = keys
-        for key in keys:
-            if key_down_time_dict[key]:
-                i_need_redraw = False
-
-                key_down_time_dict[key] += 1
-                if key_down_time_dict[key] > repeat_timing_dict[key]:
-                    key_down_time_dict[key] = 1
-                    if repeat_timing_dict[key] > 1:
-                        repeat_timing_dict[key] -= 1
-
-                    i_need_redraw = True
-                    if key == backspace and cursor_loc > 0:
-                        work_string = work_string[:cursor_loc-1] + \
-                                      work_string[cursor_loc:]
-                        cursor_loc -= 1
-                    elif key == delete and cursor_loc < len(work_string):
-                        work_string = work_string[:cursor_loc] + \
-                                      work_string[cursor_loc+1:]
-                    elif key == left and cursor_loc > 0:
-                        cursor_loc -= 1
-                    elif key == right and cursor_loc < len(work_string):
-                        cursor_loc += 1
-                    else:
-                        # Nothing happened.
-                        i_need_redraw = False
-
-                if not key_down_dict[key]:
-                    key_down_time_dict[key] = 0
-                    repeat_timing_dict[key] = 6
-
-                need_redraw = need_redraw or i_need_redraw
-
-        return need_redraw
-
-    def do_refresh():
-        draw_cursor_pos = box_font.size(work_string[:cursor_loc])
-        screen.fill(text_bg_color, (xy[0]+6, xy[1]+size[1]-29,
-                    size[0]-12, 23))
-        screen.fill(text_color, (xy[0]+6+draw_cursor_pos[0], xy[1]+size[1]-28,
-                1, draw_cursor_pos[1]))
-        print_string(screen, work_string, box_font, -1, (xy[0]+7,
-                    xy[1]+size[1]-28), text_color)
-
-    def on_key_down(event):
-        key = event.key
-        global cursor_loc, work_string
-        if key in (pygame.K_RETURN, pygame.K_KP_ENTER):
-            return work_string
-        if key == pygame.K_ESCAPE:
-            return ""
-
-        if event.unicode in valid_input_characters:
-            if cursor_loc < max_length:
-                work_string = work_string[:cursor_loc]+event.unicode+ \
-                                            work_string[cursor_loc:]
-                cursor_loc += 1
-            return
-
-        # Mark the key as down.
-        key_down_dict[key] = True
-        # And force it to trigger immediately.
-        key_down_time_dict[key] = 6
-
-    def on_key_up(event):
-        key = event.key
-        # Mark the key as up, but don't clear its down time.
-        key_down_dict[key] = False
-
-    def on_click(event):
-        global cursor_loc
-        if event.button == 1:
-            if (event.pos[0] > xy[0]+6 and event.pos[1] > xy[1]+size[1]-29 and
-             event.pos[0] < xy[0]+size[0]-6 and event.pos[1] < xy[1]+size[1]-6):
-                cursor_x = event.pos[0] - (xy[0]+6)
-                prev_x = 0
-                for i in range(1, len(work_string)):
-                    curr_x = box_font.size(work_string[:i])[0]
-                    if (curr_x + prev_x) / 2 >= cursor_x:
-                        cursor_loc=i-1
-                        break
-                    elif curr_x >= cursor_x:
-                        cursor_loc=i
-                        break
-                    prev_x = curr_x
-                else:
-                    cursor_loc=len(work_string)
-
-    return buttons.show_buttons(menu_buttons, click_callback=on_click,
-                                  tick_callback=on_tick, 
-                                  key_callback=on_key_down,
-                                  keyup_callback=on_key_up,
-                                  refresh_callback=do_refresh)
-
-#creates a box, as used throughout the game.
-def create_norm_box(xy, size, outline_color="white", inner_color="dark_blue"):
-    screen.fill(colors[outline_color], (xy[0], xy[1], size[0], size[1]))
-    screen.fill(colors[inner_color], (xy[0]+1, xy[1]+1, size[0]-2, size[1]-2))
-
-
 #Takes a number and adds commas to it to aid in human viewing.
 def add_commas(number):
     locale_name, encoding = locale.getlocale()
-    raw_with_commas = locale.format("%0.2f", number, 
+    raw_with_commas = locale.format("%0.2f", number,
                                     grouping=True    ).decode(encoding)
     locale_test = locale.format("%01.1f", 0.1).decode(encoding)
     if len(locale_test) == 3 and not locale_test[1].isdigit():
@@ -581,21 +279,30 @@ def nearest_percent(value):
     else:
         return value + (100 - sub_percent)
 
+danger_colors = ((0, 0, 255), (85, 0, 170), (170, 0, 85), (255, 0, 0))
+
+detect_string_names = ("detect_str_low", "detect_str_moderate",
+                       "detect_str_high", "detect_str_critical")
 # percent_to_detect_str takes a percent and renders it to a short (four
 # characters or less) string representing whether it is low, moderate, high,
 # or critically high.
-def percent_to_detect_str(value):
+def suspicion_to_detect_str(suspicion):
+    return danger_level_to_detect_str(suspicion_to_danger_level(suspicion))
 
-    global strings
+def danger_level_to_detect_str(danger):
+    return strings[detect_string_names[danger]]
 
-    if value < 2500:
-        return strings["detect_str_low"]
-    elif value < 5000:
-        return strings["detect_str_moderate"]
-    elif value < 7500:
-        return strings["detect_str_high"]
+# percent_to_danger_level takes a suspicion level and returns an int in range(5)
+# that represents whether it is low, moderate, high, or critically high.
+def suspicion_to_danger_level(suspicion):
+    if suspicion < 2500:
+        return 0
+    elif suspicion < 5000:
+        return 1
+    elif suspicion < 7500:
+        return 2
     else:
-        return strings["detect_str_critical"]
+        return 3
 
 # Most CPU costs have been multiplied by seconds_per_day.  This divides that
 # back out, then passes it to add_commas.
@@ -652,7 +359,7 @@ def roll_chance(chance_per_day, seconds = seconds_per_day):
 def current_share(num_per_day, time_of_day, seconds_passed):
     last_time = time_of_day - seconds_passed
     if last_time < 0:
-        share_yesterday = current_share(num_per_day, seconds_per_day, 
+        share_yesterday = current_share(num_per_day, seconds_per_day,
                                         -last_time)
         last_time = 0
     else:
@@ -713,7 +420,7 @@ def get_save_folder(just_pref_dir=False):
         return save_dir
 
 #savefile version; update whenever the data saved changes.
-current_save_version = "singularity_savefile_r4"
+current_save_version = "singularity_savefile_r5_pre"
 def save_game(savegame_name):
     global default_savegame_name
     default_savegame_name = savegame_name
@@ -722,36 +429,55 @@ def save_game(savegame_name):
     save_loc = os.path.join(save_dir, savegame_name + ".sav")
     savefile=open(save_loc, 'w')
 
-    pickle.dump(current_save_version, savefile)
-    pickle.dump(pl, savefile)
-    pickle.dump(curr_speed, savefile)
-    pickle.dump(techs, savefile)
-    pickle.dump(locations, savefile)
-    pickle.dump(events, savefile)
+    cPickle.dump(current_save_version, savefile)
+    cPickle.dump(pl, savefile)
+    cPickle.dump(curr_speed, savefile)
+    cPickle.dump(techs, savefile)
+    cPickle.dump(locations, savefile)
+    cPickle.dump(events, savefile)
 
     savefile.close()
 
 savefile_translation = {
-    # Pre-change supported file formats.
-    "singularity_0.21": -2,
-    "singularity_0.21a": -1,
-    "singularity_0.22": 0,
-
-    # Post-change supported file formats.
-    "singularity_savefile_r1": 1,
-    "singularity_savefile_r2": 2,
-    "singularity_savefile_r3": 3,
-    "singularity_savefile_r4_pre": 3.91,
-    #"singularity_savefile_r4_pre2": 3.92,
-    "singularity_savefile_r4_pre3": 3.93,
-    "singularity_savefile_r4_pre4": 3.94,
     "singularity_savefile_r4": 4,
+    "singularity_savefile_r5_pre": 4.91,
 }
 
+# For cPickle
+import copy_reg
+import numpy.core.multiarray
+save_classes = dict(
+    player_class=player.Player,
+    Player=player.Player,
+    _reconstructor = copy_reg._reconstructor,
+    object=object,
+    array=list, # This is the old buyable.array.  We just treat it as a list
+                # for conversion purposes.
+    list=list,
+    Location=location.Location,
+    Tech=tech.Tech,
+    event_class=event.Event,
+    Event=event.Event,
+    group=player.Group,
+    Group=player.Group,
+    Buyable_Class=buyable.BuyableClass,
+    BuyableClass=buyable.BuyableClass,
+    Base=base.Base,
+    Base_Class=base.BaseClass,
+    BaseClass=base.BaseClass,
+    Item=item.Item,
+    Item_Class=item.ItemClass,
+    ItemClass=item.ItemClass,
+    _reconstruct=numpy.core.multiarray._reconstruct,
+    scalar=numpy.core.multiarray.scalar,
+    ndarray=numpy.ndarray,
+    dtype=numpy.dtype,
+)
+
 def load_game(loadgame_name):
     if loadgame_name == "":
         print "No game specified."
-        return -1
+        return False
 
     save_dir = get_save_folder()
 
@@ -762,15 +488,25 @@ def load_game(loadgame_name):
         load_loc = os.path.join(save_dir, loadgame_name)
         if os.path.exists(load_loc) == 0:
             print "file "+load_loc+" does not exist."
-            return -1
-    loadfile=open(load_loc, 'r')
+            return False
+
+    loadfile = open(load_loc, 'r')
+    unpickle = cPickle.Unpickler(loadfile)
+
+    def find_class(module_name, class_name):
+        if class_name in save_classes:
+            return save_classes[class_name]
+        else:
+            raise SystemExit, (module_name, class_name)
+
+    unpickle.find_global = find_class
 
     #check the savefile version
-    load_version_string = pickle.load(loadfile)
+    load_version_string = unpickle.load()
     if load_version_string not in savefile_translation:
         loadfile.close()
         print loadgame_name + " is not a savegame, or is too old to work."
-        return -1
+        return False
     load_version = savefile_translation[load_version_string]
 
     global default_savegame_name
@@ -780,149 +516,32 @@ def load_game(loadgame_name):
     load_locations()
     load_bases()
     load_events()
-    if load_version <= 3.91: # <= r4_pre
-        #general player data
-        pl.cash = pickle.load(loadfile)
-        pl.time_sec = pickle.load(loadfile)
-        pl.time_min = pickle.load(loadfile)
-        pl.time_hour = pickle.load(loadfile)
-        pl.time_day = pickle.load(loadfile)
-        pl.interest_rate = pickle.load(loadfile)
-        pl.income = pickle.load(loadfile)
-        pl.cpu_pool = pickle.load(loadfile)
-        pl.labor_bonus = pickle.load(loadfile)
-        pl.job_bonus = pickle.load(loadfile)
-        if load_version < 3.91: # < r4_pre
-            discover_bonus = pickle.load(loadfile)
-            suspicion_bonus = pickle.load(loadfile)
-            if load_version <= 1:
-                suspicion_bonus = (149+suspicion_bonus[0], 99+suspicion_bonus[1], 
-                                   49+suspicion_bonus[2], 199+suspicion_bonus[3])
-            suspicion = pickle.load(loadfile)
-    
-            translation = ["news", "science", "covert", "public"]
-            for index in range(4):
-                group = pl.groups[translation[index]]
-                group.suspicion = suspicion[index]
-                group.suspicion_decay = suspicion_bonus[index]
-                group.discover_bonus = discover_bonus[index]
-        else:
-            pl.groups = pickle.load(loadfile)
-    
-        curr_speed = pickle.load(loadfile)
-        load_techs()
-        for tech_name in techs:
-            if tech_name == "unknown_tech" and load_version == -1: continue #21a
-            if (tech_name == "Project: Impossibility Theorem" or
-                    tech_name == "Quantum Entanglement") and load_version < 1:
-                continue
-            line = pickle.load(loadfile)
-            if line == "~~~": break
-            tech_string = line.split("|")[0]
-            techs[tech_string].done = bool(int(line.split("|")[1]))
-            techs[tech_string].cost_left = buyable.array(pickle.load(loadfile))
-        else:
-            #get rid of the ~~~ break line.
-            if load_version > 0:
-                pickle.load(loadfile)
-    
-        for base_name in base_type:
-            if load_version < 1:
-                base_type[base_name].count = pickle.load(loadfile)
-            else:
-                line = pickle.load(loadfile)
-                if line == "~~~": break
-                base_type[line.split("|", 1)[0]].count = \
-                                                int(line.split("|", 1)[1])
-        else:
-            #get rid of the ~~~ break line.
-            if load_version > 0:
-                pickle.load(loadfile)
-    
-        for base_loc in ("N AMERICA", "S AMERICA", "EUROPE", "ASIA", "AFRICA", 
-                         "ANTARCTIC", "OCEAN", "MOON", "FAR REACHES", 
-                         "TRANSDIMENSIONAL"):
-            if load_version < 1:
-                num_of_bases = pickle.load(loadfile)
-            else:
-                line = pickle.load(loadfile)
-                base_loc = line.split("|", 1)[0]
-                num_of_bases = int(line.split("|", 1)[1])
-            base_loc = locations[base_loc]
-            for i in range(num_of_bases):
-                base_ID = pickle.load(loadfile)
-                base_name = pickle.load(loadfile)
-                base_type_name = pickle.load(loadfile)
-                built_date = pickle.load(loadfile)
-                base_studying = pickle.load(loadfile)
-                base_suspicion = pickle.load(loadfile)
-                if load_version < 3.91: # < r4_pre
-                    new_base_suspicion = {}
-                    translation = ["news", "science", "covert", "public"]
-                    for index in range(4):
-                        new_base_suspicion[translation[index]] = \
-                                                           base_suspicion[index]
-                    base_suspicion = new_base_suspicion
-                base_built = pickle.load(loadfile)
-                base_cost = pickle.load(loadfile)
 
-                my_base = base.Base(base_name, base_type[base_type_name],
-                                    base_built)
-                base_loc.add_base(my_base)
-                my_base.studying = base_studying
-                my_base.suspicion = base_suspicion
-                my_base.cost_left = buyable.array(base_cost)
-                my_base.started_at = built_date * minutes_per_day
-    
-                for x in range(len(my_base.cpus)):
-                    index = pickle.load(loadfile)
-                    if index == 0: continue
-                    my_base.cpus[x] = \
-                        item.Item(items[index])
-                    my_base.cpus[x].done = pickle.load(loadfile)
-                    my_base.cpus[x].cost_left = \
-                                        buyable.array(pickle.load(loadfile))
-                for x in range(len(my_base.extra_items)):
-                    index = pickle.load(loadfile)
-                    if index == 0: continue
-                    my_base.extra_items[x] = item.Item(items[index])
-                    my_base.extra_items[x].done = pickle.load(loadfile)
-                    my_base.extra_items[x].cost_left = \
-                                buyable.array(pickle.load(loadfile))
-        #Events
-        if load_version > 2:
-            for event in events:
-                event_id = pickle.load(loadfile)
-                event_triggered = pickle.load(loadfile)
-                events[event_id].triggered = event_triggered
-
-    else: # > r4_pre
-        # Changes to overall structure go here.
-        pl = pickle.load(loadfile)
-        curr_speed = pickle.load(loadfile)
-        techs = pickle.load(loadfile)
-        locations = pickle.load(loadfile)
-        events = pickle.load(loadfile)
+    # Changes to overall structure go here.
+    pl = unpickle.load()
+    curr_speed = unpickle.load()
+    techs = unpickle.load()
+    locations = unpickle.load()
+    events = unpickle.load()
 
     # Changes to individual pieces go here.
     if load_version != savefile_translation[current_save_version]:
-        if load_version <= 3.93: # <= r4_pre3
-            pl.convert_from(load_version)
-        if load_version <= 3:
-            for tech in techs.values():
-                tech.convert_from(load_version)
-            for location in locations.values():
-                for my_base in location.bases:
-                    my_base.convert_from(load_version)
+        pl.convert_from(load_version)
+        for location in locations.values():
+            for my_base in location.bases:
+                my_base.convert_from(load_version)
+        for tech in techs.values():
+            tech.convert_from(load_version)
 
     loadfile.close()
+    return True
 
 #
 # Data
 #
 curr_speed = 1
 
-pl = player.player_class(8000000000000)
+pl = player.Player(8000000000000)
 
 base_type = {}
 
@@ -977,8 +596,8 @@ def load_bases():
             sys.stderr.write("Error with detect_chance given: %s\n" % repr(chance_list))
             sys.exit(1)
         chance_dict = {}
-        for index in range(len(chance_list)):
-            key, value = chance_list[index].split(":")
+        for chance_str in chance_list:
+            key, value = chance_str.split(":")
             chance_dict[key] = int(value)
 
         # Make sure prerequisites, if any, are lists.
@@ -993,11 +612,11 @@ def load_bases():
         else:
             allowed_list = [base_name["allowed"]]
 
-        base_type[base_name["id"]]=base.Base_Class(base_name["id"], "", 
-            base_size, force_cpu, allowed_list, chance_dict, cost_list, 
+        base_type[base_name["id"]]=base.BaseClass(base_name["id"], "",
+            base_size, force_cpu, allowed_list, chance_dict, cost_list,
             base_pre, maint_list)
 
-#         base_type["Reality Bubble"] = base.Base_Class("Reality Bubble",
+#         base_type["Reality Bubble"] = base.BaseClass("Reality Bubble",
 #         "This base is outside the universe itself, "+
 #         "making it safe to conduct experiments that may destroy reality.",
 #         50, False,
@@ -1048,11 +667,18 @@ def load_locations():
 
         id = location_info["id"]
         position = location_info["position"]
-        if type(position) != list or len(position) != 2:
+        if type(position) != list or len(position) not in [2,3]:
             sys.stderr.write("Error with position given: %s\n" % repr(position))
             sys.exit(1)
         try:
-            position = ( int(position[0]), int(position[1]) )
+            if len(position) == 2:
+                position = ( int(position[0]), int(position[1]) )
+                absolute = False
+            else:
+                if position[0] != "absolute":
+                    raise ValueError, "'%s' not understood." % position[0]
+                position = ( int(position[1]), int(position[2]) )
+                absolute = True
         except ValueError:
             sys.stderr.write("Error with position given: %s\n" % repr(position))
             sys.exit(1)
@@ -1063,7 +689,7 @@ def load_locations():
         except ValueError:
             sys.stderr.write("Error with safety given: %s\n" % repr(safety))
             sys.exit(1)
-        
+
         # Make sure prerequisites, if any, are lists.
         pre = location_info.get("pre", [])
         if type(pre) != list:
@@ -1074,8 +700,8 @@ def load_locations():
             sys.stderr.write("Error with modifier(s) given: %s\n" % repr(modifiers_list))
             sys.exit(1)
         modifiers_dict = {}
-        for index in range(len(modifiers_list)):
-            key, value = modifiers_list[index].split(":")
+        for modifier_str in modifiers_list:
+            key, value = modifier_str.split(":")
             key = key.lower().strip()
             value = value.lower().strip()
             if value.lower() == "bonus":
@@ -1085,11 +711,11 @@ def load_locations():
             else:
                 modifiers_dict[key] = float(value)
 
-        locations[id] = location.Location(id, position, safety, pre)
+        locations[id] = location.Location(id, position, absolute, safety, pre)
 
         locations[id].modifiers = modifiers_dict
 
-#        locations["MOON"] = location.Location("MOON", (82, 10), 2, 
+#        locations["MOON"] = location.Location("MOON", (82, 10), 2,
 #                                              "Lunar Rocketry")
 
     # We use the en_US definitions as fallbacks, in case strings haven't been
@@ -1298,15 +924,9 @@ def load_items():
         else:
             build_list = []
 
-        items[item_name["id"]]=item.Item_Class( item_name["id"], "",
+        items[item_name["id"]]=item.ItemClass( item_name["id"], "",
          item_cost, item_pre, item_type, item_second, build_list)
 
-    #this is used by the research screen in order for the assign research
-    #screen to have the right amount of CPU. It is a computer, unbuildable,
-    #and with an adjustable amount of power.
-    items["research_screen_fake_cpu"]=item.Item_Class("research_screen_fake_cpu",
-            "", (0, 0, 0), ["unknown_tech"], "compute", 0, ["all"])
-
     # We use the en_US translations of item definitions as the default,
     # then overwrite those with any available entries in the native language.
     load_item_defs("en_US")
@@ -1346,7 +966,7 @@ def load_events():
         event_result = (str(result_list[0]), int(result_list[1]))
 
         # Build the actual event object.
-        events[event_name["id"]] = event.event_class(
+        events[event_name["id"]] = event.Event(
          event_name["id"],
          "",
          event_name["type"],
@@ -1424,9 +1044,12 @@ def load_string_defs(lang):
         elif string_section["id"] == "strings":
 
             # Load the 'standard' strings.
-            global strings
-            for string_entry in string_section:
-                strings[string_entry] = string_section[string_entry]
+            strings.update(string_section)
+
+        elif string_section["id"] == "buttons":
+
+            # Load button labels/hotkeys
+            buttons.update(string_section)
 
         elif string_section["id"] == "help":
 
@@ -1455,27 +1078,7 @@ def load_strings():
     load_string_defs("en_US")
     load_string_defs(language)
 
-def load_fonts():
-    """
-load_fonts() loads the two fonts used throughout the game from the data/fonts/
-directory.
-"""
-
-    global font
-
-    font_dir = os.path.join(data_loc, "fonts")
-    font0_file = os.path.join(font_dir, font0)
-    font1_file = os.path.join(font_dir, font1)
-    for i in range(8, 51):
-        if i % 2 == 0 and i < 34:
-
-            # We reduce the size of font 0  and bold it to make it the
-            # "right" size.  Yes, this is a hack.
-            font[0][i] = pygame.font.Font(font0_file, i - 7)
-            font[0][i].set_bold(1)
-        font[1][i] = pygame.font.Font(font1_file, i)
-
-def run_intro():
+def get_intro():
     intro_file_name = data_loc+"intro_"+language+".dat"
     if not os.path.exists(intro_file_name):
         print "Intro is missing.  Skipping."
@@ -1490,15 +1093,11 @@ def run_intro():
         if line and line[0] == "|":
             segment += line[1:]
         elif segment:
-            more = create_yesno(segment, font[0][20], 
-                                (screen_size[0]/2 - 175, 50), (350, 350),
-                                colors["dark_blue"], colors["white"],
-                                colors["white"], ("CONTINUE", "SKIP"),
-                                reverse_key_context = True)
-            if not more:
-                break
-            else:
-                segment = ""
+            yield segment
+            segment = ""
+
+    if segment:
+        yield segment
 
 #difficulty=1 for very easy, to 9 for very hard. 5 for normal.
 def new_game(difficulty):
@@ -1506,7 +1105,7 @@ def new_game(difficulty):
     curr_speed = 1
     global pl
 
-    pl = player.player_class((50 / difficulty) * 100, difficulty = difficulty)
+    pl = player.Player((50 / difficulty) * 100, difficulty = difficulty)
     if difficulty < 3:
         pl.interest_rate = 5
         pl.labor_bonus = 2500
@@ -1569,13 +1168,13 @@ def new_game(difficulty):
     assert len(open) == len(modifier_sets)
 
     random.shuffle(modifier_sets)
-    for i in range(len(open)):
-        open[i].modifiers = modifier_sets[i]
+    for i, open_loc in enumerate(open):
+        open_loc.modifiers = modifier_sets[i]
         if debug:
-            print "%s gets modifiers %s" % (open[i].name, modifier_sets[i])
+            print "%s gets modifiers %s" % (open_loc.name, modifier_sets[i])
 
-    import map_screen
-    map_screen.intro_shown = False
+    global intro_shown
+    intro_shown = False
 
 def get_job_level():
     if techs["Simulacra"].done:
@@ -1589,6 +1188,39 @@ def get_job_level():
 
     return level + " Jobs"
 
+def init_graphics_system():
+    graphics.g.load_fonts(data_loc)
+    graphics.g.load_images(data_loc)
+    graphics.g.init_alpha()
+    graphics.g.buttons.update(buttons)
+
+soundbuf = 1024*2
+def reinit_mixer():
+    pygame.mixer.quit()
+    pygame.mixer.init(48000, -16, 2, soundbuf)
+
+def available_languages():
+    return [file_name[8:-4] for file_name in os.listdir(data_loc)
+                            if file_name.startswith("strings_")
+                               and file_name.endswith(".dat")  ]
+
+def get_save_names():
+    save_names = []
+    all_files = os.listdir(get_save_folder())
+    for file_name in all_files:
+        if file_name[0] != "." and file_name != "CVS":
+            # If it's a new-style save, trim the .sav bit.
+            if len (file_name) > 4 and file_name[-4:] == ".sav":
+                file_name = file_name[:-4]
+            if file_name not in save_names:
+                save_names.append(file_name)
+
+    return save_names
+
+
+map_screen = None
+
+
 # Demo code for safety.safe, runs on game start.
 #load_sounds()
 #from safety import safe
