@@ -20,26 +20,31 @@
 #This file contains the player class.
 
 import random
+from operator import truediv
+from numpy import array
 
 import g
+from graphics import g as gg
 import buyable
 from buyable import cash, cpu, labor
-import finance_screen
 
-class group(object):
+group_list = ("news", "science", "covert", "public")
+class Group(object):
     discover_suspicion = 1000
-    def __init__(self, name, suspicion = 0, suspicion_decay = 100, 
+    def __init__(self, name, suspicion = 0, suspicion_decay = 100,
                  discover_bonus = 10000):
         self.name = name
         self.suspicion = suspicion
         self.suspicion_decay = suspicion_decay
         self.discover_bonus = discover_bonus
 
-    def new_day(self):
+    def decay_rate(self):
         # Suspicion reduction is now quadratic.  You get a certain percentage
         # reduction, or a base .01% reduction, whichever is better.
-        quadratic_down = (self.suspicion * self.suspicion_decay) / 10000
-        self.alter_suspicion(-max(quadratic_down, 1))
+        return max(1, (self.suspicion * self.suspicion_decay) // 10000)
+
+    def new_day(self):
+        self.alter_suspicion(-self.decay_rate())
 
     def alter_suspicion(self, change):
         self.suspicion = max(self.suspicion + change, 0)
@@ -53,7 +58,30 @@ class group(object):
     def discovered_a_base(self):
         self.alter_suspicion(self.discover_suspicion)
 
-class player_class(object):
+    def detects_per_day_to_danger_level(self, detects_per_day):
+        raw_suspicion_per_day = detects_per_day * self.discover_suspicion
+        suspicion_per_day = raw_suspicion_per_day - self.decay_rate()
+
+        # +1%/day or death within 10 days
+        if suspicion_per_day > 100 \
+           or (self.suspicion + suspicion_per_day * 10) >= 10000:
+            return 3
+        # +0.5%/day or death within 100 days
+        elif suspicion_per_day > 50 \
+           or (self.suspicion + suspicion_per_day * 100) >= 10000:
+            return 2
+        # Suspicion increasing.
+        elif suspicion_per_day > 0:
+            return 1
+        # Suspicion steady or decreasing.
+        else:
+            return 0
+
+class DryRunInfo(object):
+    pass
+
+class Player(object):
+    intro_shown = False
     def __init__(self, cash, time_sec=0, time_min=0, time_hour=0, time_day=0,
                  difficulty = 5):
         self.difficulty = difficulty
@@ -68,6 +96,7 @@ class player_class(object):
             self.had_grace = True
         else:
             self.had_grace = self.in_grace_period()
+        self.apotheosis = False
 
         self.cash = cash
         self.interest_rate = 1
@@ -79,32 +108,26 @@ class player_class(object):
 
         self.partial_cash = 0
 
-        self.groups = {"news":    group("news",    suspicion_decay = 150),
-                       "science": group("science", suspicion_decay = 100),
-                       "covert":  group("covert",  suspicion_decay =  50),
-                       "public":  group("public",  suspicion_decay = 200)}
+        self.groups = {"news":    Group("news",    suspicion_decay = 150),
+                       "science": Group("science", suspicion_decay = 100),
+                       "covert":  Group("covert",  suspicion_decay =  50),
+                       "public":  Group("public",  suspicion_decay = 200)}
 
         self.grace_multiplier = 200
         self.last_discovery = self.prev_discovery = ""
 
-        self.maintenance_cost = buyable.array((0,0,0))
+        self.maintenance_cost = array((0,0,0), long)
 
-        self.have_cpu = True
-        self.complete_bases = 1
-        self.complex_bases = 0
+        self.cpu_usage = {}
+        self.available_cpus = [1, 0, 0, 0, 0]
+        self.sleeping_cpus = 0
 
     def convert_from(self, old_version):
-         if old_version <= 3.94: # <= r4_pre4
-             # We don't know what the difficulty was, and techs have fooled with
-             # any values that would help (not to mention the headache of 
-             # different versions.  So we set it to Very Easy, which means that
-             # they shouldn't be hurt by any new mechanics.
-             self.difficulty = 1
-         if old_version <= 3.93: # <= r4_pre3
-             self.grace_multiplier = int(1000000./float(self.labor_bonus))
-         if old_version <= 3.91: # <= r4_pre
-             self.make_raw_times()
-             self.had_grace = self.in_grace_period()
+         if old_version < 4.91: # < r5_pre
+             self.cpu_usage = {}
+             self.apotheosis = g.techs["Apotheosis"].done
+             if self.apotheosis:
+                 self.had_grace = True
 
     def make_raw_times(self):
         self.raw_hour = self.time_day * 24 + self.time_hour
@@ -113,15 +136,10 @@ class player_class(object):
         self.raw_day = self.time_day
 
     def update_times(self):
-        # Total minutes/hours/days spent.
-        self.raw_min = self.raw_sec // 60
-        self.raw_hour = self.raw_min // 60
-        self.raw_day = self.raw_hour // 24
-
-        # Remainders.
-        self.time_sec = self.raw_sec % 60
-        self.time_min = self.raw_min % 60
-        self.time_hour = self.raw_hour % 24
+        # Total time,  display time
+        self.raw_min,  self.time_sec  = divmod(self.raw_sec, 60)
+        self.raw_hour, self.time_min  = divmod(self.raw_min, 60)
+        self.raw_day,  self.time_hour = divmod(self.raw_hour, 24)
 
         # Overflow
         self.time_day = self.raw_day
@@ -135,11 +153,14 @@ class player_class(object):
     def do_jobs(self, cpu_time):
         earned, self.partial_cash = self.get_job_info(cpu_time)
         self.cash += earned
+        return earned
 
     def get_job_info(self, cpu_time, partial_cash = None):
         if partial_cash == None:
             partial_cash = self.partial_cash
 
+        assert partial_cash >= 0
+
         cash_per_cpu = g.jobs[g.get_job_level()][0]
         if g.techs["Advanced Simulacra"].done:
             #10% bonus income
@@ -152,12 +173,11 @@ class player_class(object):
 
         return cash, new_partial_cash
 
-    def give_time(self, time_sec):
+    def give_time(self, time_sec, dry_run=False):
         if time_sec == 0:
-            return
-
-        needs_refresh = False
+            return 0
 
+        old_time = self.raw_sec
         last_minute = self.raw_min
         last_day = self.raw_day
 
@@ -184,109 +204,44 @@ class player_class(object):
 
         time_of_day = g.pl.raw_sec % g.seconds_per_day
 
+        old_cash = self.cash
+        old_partial_cash = self.partial_cash
+
         techs_researched = []
         bases_constructed = []
-        cpus_constructed = {}
+        cpus_constructed = []
         items_constructed = []
 
         bases_under_construction = []
         items_under_construction = []
         self.cpu_pool = 0
-        self.have_cpu = False
-        self.complete_bases = 0
-        self.complex_bases = 0
 
-        # Re-calculate the maintenance.
-        self.maintenance_cost = buyable.array( (0,0,0) )
-
-        # Phase 1: Collect CPU and construction info.
-        #          Spend CPU, then Cash/Labor.
+        # Collect base info, including maintenance.
+        self.maintenance_cost = array( (0,0,0), long )
         for base in g.all_bases():
             if not base.done:
                 bases_under_construction.append(base)
             else:
-                self.complete_bases += 1
-                if base.is_complex():
-                    self.complex_bases += 1
-
-                unfinished_cpus = [(base, item) for item in base.cpus 
-                                                if item and not item.done]
-                unfinished_items = [(base, item) for item in base.extra_items 
+                if base.cpus is not None and not base.cpus.done:
+                    items_under_construction += [(base, base.cpus)]
+                unfinished_items = [(base, item) for item in base.extra_items
                                                  if item and not item.done]
-                items_under_construction += unfinished_cpus + unfinished_items
+                items_under_construction += unfinished_items
 
                 self.maintenance_cost += base.maintenance
 
-                if base.power_state != "Stasis":
-                    cpu_power = base.processor_time() * secs_passed
-                    self.have_cpu = self.have_cpu or cpu_power
-                    if base.power_state != "Active":
-                        continue
+        # Maintenence?  Gods don't need no steenking maintenance!
+        if self.apotheosis:
+            self.maintenance_cost = array( (0,0,0), long )
 
-                    if base.studying in g.jobs:
-                        self.do_jobs(cpu_power)
-                        continue
+        # Any CPU explicitly assigned to jobs earns its dough.
+        job_cpu = self.cpu_usage.get("jobs", 0) * secs_passed
+        explicit_job_cash = self.do_jobs(job_cpu)
 
-                    # Everything else goes into the CPU pool.  Research goes 
-                    # through it for simplicity and to allow spill-over.
-                    self.cpu_pool += cpu_power
-
-                    if base.studying in g.techs:
-                        tech = g.techs[base.studying]
-                        # Note that we restrict the CPU available to prevent
-                        # the tech from pulling from the rest of the CPU pool.
-                        tech_gained = tech.work_on(cash_available=0, 
-                                                   cpu_available=cpu_power)
-                        if tech_gained:
-                            techs_researched.append(tech)
-
-                    # Explicit and implicit assignment to the CPU pool was
-                    # already handled.
-
-        # Maintenance CPU.
-        if self.maintenance_cost[cpu] > self.cpu_pool:
-            self.maintenance_cost[cpu] -= self.cpu_pool
-            self.cpu_pool = 0
-        else:
-            self.cpu_pool -= self.maintenance_cost[cpu]
-            self.maintenance_cost[cpu] = 0
-
-        # Construction CPU.
-        # Bases.
-        for base in bases_under_construction:
-            built_base = base.work_on(cash_available = 0)
-
-            if built_base:
-                bases_constructed.append(base)
-
-        # Items.
-        for base, item in items_under_construction:
-            built_item = item.work_on(cash_available = 0)
-
-            if built_item:
-                # Non-CPU items.
-                if item.item_type != "compute":
-                    items_constructed.append( (base, item) )
-                # CPUs.
-                else:
-                    cpus_constructed.setdefault(base, 0)
-                    cpus_constructed[base] += 1
-
-        # Jobs.
-        if self.cpu_pool > 0:
-            self.do_jobs(self.cpu_pool)
-            self.cpu_pool = 0
-
-        # And now we get to spend cash and labor.
-        # Research.
-        for tech in g.techs.values():
-            tech_gained = tech.work_on(time = mins_passed)
-            if tech_gained:
-                techs_researched.append(tech)
-
-        # Maintenance.
-        cash_maintenance = g.current_share(self.maintenance_cost[cash],
+        # Pay maintenance cash, if we can.
+        cash_maintenance = g.current_share(int(self.maintenance_cost[cash]),
                                            time_of_day, secs_passed)
+        full_cash_maintenance = cash_maintenance
         if cash_maintenance > self.cash:
             cash_maintenance -= self.cash
             self.cash = 0
@@ -294,113 +249,216 @@ class player_class(object):
             self.cash -= cash_maintenance
             cash_maintenance = 0
 
-        # Construction.
-        # Bases.
+        tech_cpu = 0
+        tech_cash = 0
+        # Do research, fill the CPU pool.
+        default_cpu = self.available_cpus[0]
+        for task, cpu_assigned in self.cpu_usage.iteritems():
+            if cpu_assigned == 0:
+                continue
+
+            default_cpu -= cpu_assigned
+            real_cpu = cpu_assigned * secs_passed
+            if task != "jobs":
+                self.cpu_pool += real_cpu
+                if task != "cpu_pool":
+                    if dry_run:
+                        spent = g.techs[task].calculate_work(time=mins_passed,
+                                                     cpu_available=real_cpu)[0]
+                        g.pl.cpu_pool -= int(spent[cpu])
+                        g.pl.cash -= int(spent[cash])
+                        tech_cpu += cpu_assigned
+                        tech_cash += int(spent[cash])
+                        continue
+
+                    # Note that we restrict the CPU available to prevent
+                    # the tech from pulling from the rest of the CPU pool.
+                    tech_gained = g.techs[task].work_on(cpu_available=real_cpu,
+                                                        time=mins_passed)
+                    if tech_gained:
+                        techs_researched.append(g.techs[task])
+        self.cpu_pool += default_cpu * secs_passed
+
+        # And now we use the CPU pool.
+        # Maintenance CPU.
+        cpu_maintenance = self.maintenance_cost[cpu] * secs_passed
+        if cpu_maintenance > self.cpu_pool:
+            cpu_maintenance -= self.cpu_pool
+            self.cpu_pool = 0
+        else:
+            self.cpu_pool -= int(cpu_maintenance)
+            cpu_maintenance = 0
+
+        construction_cpu = 0
+        construction_cash = 0
+        # Base construction.
         for base in bases_under_construction:
+            if dry_run:
+                spent = base.calculate_work(time=mins_passed,
+                                            cpu_available=real_cpu )[0]
+                g.pl.cpu_pool -= int(spent[cpu])
+                g.pl.cash -= int(spent[cash])
+                construction_cpu += int(spent[cpu])
+                construction_cash += int(spent[cash])
+                continue
+
             built_base = base.work_on(time = mins_passed)
 
             if built_base:
                 bases_constructed.append(base)
 
-        # Items.
+        # Item construction.
         for base, item in items_under_construction:
+            if dry_run:
+                spent = item.calculate_work(time=mins_passed,
+                                            cpu_available=0 )[0]
+                g.pl.cpu_pool -= int(spent[cpu])
+                g.pl.cash -= int(spent[cash])
+                construction_cpu += int(spent[cpu])
+                construction_cash += int(spent[cash])
+                continue
+
             built_item = item.work_on(time = mins_passed)
 
             if built_item:
                 # Non-CPU items.
-                if item.type.item_type != "compute":
+                if item.type.item_type != "cpu":
                     items_constructed.append( (base, item) )
                 # CPUs.
                 else:
-                    cpus_constructed.setdefault(base, 0)
-                    cpus_constructed[base] += 1
+                    cpus_constructed.append( (base, item) )
 
-        # Are we still in the grace period?
-        grace = self.in_grace_period(self.had_grace, self.complete_bases, 
-                                     self.complex_bases)
+        # Jobs via CPU pool.
+        pool_job_cash = 0
+        if self.cpu_pool > 0:
+            pool_job_cash = self.do_jobs(self.cpu_pool)
+
+        # Second attempt at paying off our maintenance cash.
+        if cash_maintenance > self.cash:
+            # In the words of Scooby Doo, "Ruh roh."
+            cash_maintenance -= self.cash
+            self.cash = 0
+        else:
+            # Yay, we made it!
+            self.cash -= cash_maintenance
+            cash_maintenance = 0
+
+        # Exit point for a dry run.
+        if dry_run:
+            # Collect the cash information.
+            cash_info = DryRunInfo()
+
+            cash_info.interest = self.get_interest()
+            cash_info.income = self.income
+            self.cash += cash_info.interest + cash_info.income
+
+            cash_info.explicit_jobs = explicit_job_cash
+            cash_info.pool_jobs = pool_job_cash
+            cash_info.jobs = explicit_job_cash + pool_job_cash
+
+            cash_info.tech = tech_cash
+            cash_info.construction = construction_cash
+
+            cash_info.maintenance_needed = full_cash_maintenance
+            cash_info.maintenance_shortfall = cash_maintenance
+            cash_info.maintenance = full_cash_maintenance - cash_maintenance
+
+            cash_info.start = old_cash
+            cash_info.end = self.cash
+
+
+            # Collect the CPU information.
+            cpu_info = DryRunInfo()
+
+            cpu_info.available = self.available_cpus[0]
+            cpu_info.sleeping = self.sleeping_cpus
+            cpu_info.total = cpu_info.available + cpu_info.sleeping
+
+            cpu_info.tech = tech_cpu
+            cpu_info.construction = construction_cpu
+
+            cpu_info.maintenance_needed = self.maintenance_cost[cpu]
+            cpu_info.maintenance_shortfall = cpu_maintenance
+            cpu_info.maintenance = cpu_info.maintenance_needed \
+                                   - cpu_info.maintenance_shortfall
+
+            cpu_info.explicit_jobs = self.cpu_usage.get("jobs", 0)
+            cpu_info.pool_jobs = self.cpu_pool / float(time_sec)
+            cpu_info.jobs = self.cpu_usage.get("jobs", 0) + cpu_info.pool_jobs
+
+            cpu_info.explicit_pool = self.cpu_usage.get("cpu_pool", 0)
+            cpu_info.default_pool = default_cpu
+            cpu_info.pool = self.cpu_usage.get("cpu_pool", 0) + default_cpu
+
+            # Restore the old state.
+            self.cash = old_cash
+            self.partial_cash = old_partial_cash
+            self.raw_sec = old_time
+            self.update_times()
+
+            return (cash_info, cpu_info)
 
-        # Phase 2: Dialogs, maintenance, and discovery.
         # Tech gain dialogs.
         for tech in techs_researched:
+            del self.cpu_usage[tech.id]
             text = g.strings["tech_gained"] % \
-                   {"tech": tech.name, 
+                   {"tech": tech.name,
                     "tech_message": tech.result}
-            g.create_dialog(text)
-            g.curr_speed = 0
-            needs_refresh = 1
+            self.pause_game()
+            g.map_screen.show_message(text)
 
         # Base complete dialogs.
         for base in bases_constructed:
             text = g.strings["construction"] % {"base": base.name}
-            g.curr_speed = 0
-            needs_refresh = 1
-            g.create_dialog(text)
+            self.pause_game()
+            g.map_screen.show_message(text)
 
             if base.type.id == "Stolen Computer Time" and \
-                    base.cpus[0].type.id == "Gaming PC":
+                    base.cpus.type.id == "Gaming PC":
                 text = g.strings["lucky_hack"] % {"base": base.name}
-                g.create_dialog(text)
+                g.map_screen.show_message(text)
 
         # CPU complete dialogs.
-        for base, new_cpus in cpus_constructed.iteritems():
-            if new_cpus == len(base.cpus):
-                finished_cpus = new_cpus
-            else:
-                finished_cpus = len([item for item in base.cpus 
-                                          if item and item.done  ])
-
-            if finished_cpus == len(base.cpus): # Finished all the CPUs.
+        for base, cpus in cpus_constructed:
+            if base.cpus.count == base.type.size: # Finished all CPUs.
                 text = g.strings["item_construction_single"] % \
-                       {"item": base.cpus[0].type.name, "base": base.name}
-                g.create_dialog(text)
-                g.curr_speed = 0
-                needs_refresh = 1
-            elif finished_cpus == new_cpus: # Finished the first batch of CPUs.
+                       {"item": base.cpus.type.name, "base": base.name}
+            else: # Just finished this batch of CPUs.
                 text = g.strings["item_construction_batch"] % \
-                       {"item": base.cpus[0].type.name, "base": base.name}
-                g.create_dialog(text)
-                g.curr_speed = 0
-                needs_refresh = 1
-            else:
-                pass # No message unless we just finished the first or last CPU.
-            
+                       {"item": base.cpus.type.name, "base": base.name}
+            self.pause_game()
+            g.map_screen.show_message(text)
+
         # Item complete dialogs.
         for base, item in items_constructed:
             text = g.strings["item_construction_single"] % \
                    {"item": item.type.name, "base": base.name}
-            g.create_dialog(text)
-            g.curr_speed = 0
-            needs_refresh = 1
+            self.pause_game()
+            g.map_screen.show_message(text)
+
+        # Are we still in the grace period?
+        grace = self.in_grace_period(self.had_grace)
 
         # If we just lost grace, show the warning.
         if self.had_grace and not grace:
             self.had_grace = False
 
-            g.create_dialog(g.strings["grace_warning"])
-            needs_refresh = 1
-            g.curr_speed = 0
+            self.pause_game()
+            g.map_screen.show_message(g.strings["grace_warning"])
 
-        # Discovery, clear finished techs.
+        # Maintenance death, discovery.
         dead_bases = []
         for base in g.all_bases():
-            if not grace and not base.has_grace():
-                detect_chance = base.get_detect_chance()
-                if g.debug:
-                    print "Chance of discovery for base %s: %s" % \
-                        (base.name, repr(detect_chance))
-                for group, chance in detect_chance.iteritems():
-                    if g.roll_chance(chance/10000., secs_passed):
-                        dead_bases.append( (base, group) )
-                        break
+            dead = False
 
             # Maintenance deaths.
             if base.done:
-                if self.maintenance_cost[cpu] and base.maintenance[cpu]:
-                    self.maintenance_cost[cpu] = \
-                        max(0, self.maintenance_cost[cpu] 
-                                   - base.maintenance[cpu])
+                if cpu_maintenance and base.maintenance[cpu]:
+                    refund = base.maintenance[cpu] * secs_passed
+                    cpu_maintenance = max(0, cpu_maintenance - refund)
+
                     #Chance of base destruction if cpu-unmaintained: 1.5%
-                    if g.roll_percent(150):
+                    if not dead and g.roll_chance(.015, secs_passed):
                         dead_bases.append( (base, "maint") )
                         dead = True
 
@@ -410,42 +468,79 @@ class player_class(object):
                     if base_needs:
                         cash_maintenance = max(0, cash_maintenance - base_needs)
                         #Chance of base destruction if cash-unmaintained: 1.5%
-                        if g.roll_percent(150):
+                        if not dead and g.roll_chance(.015, secs_passed):
                             dead_bases.append( (base, "maint") )
                             dead = True
-                
-            if base.studying in g.techs and g.techs[base.studying].done:
-                base.studying = ""
 
-        if self.remove_bases(dead_bases):
-            needs_refresh = 1
+            # Discoveries
+            if not (grace or dead or base.has_grace()):
+                detect_chance = base.get_detect_chance()
+                if g.debug:
+                    print "Chance of discovery for base %s: %s" % \
+                        (base.name, repr(detect_chance))
+
+                for group, chance in detect_chance.iteritems():
+                    if g.roll_chance(chance/10000., secs_passed):
+                        dead_bases.append( (base, group) )
+                        dead = True
+                        break
+
+        # Base disposal and dialogs.
+        self.remove_bases(dead_bases)
 
         # Random Events
-        for event in g.events:
-            if g.roll_chance(g.events[event].chance/10000., time_sec):
-                #Skip events already flagged as triggered.
-                if g.events[event].triggered == 1:
-                    continue
-                g.events[event].trigger()
-                needs_refresh = 1
-                break # Don't trigger more than one at a time.
+        if not grace:
+            for event in g.events:
+                if g.roll_chance(g.events[event].chance/10000., time_sec):
+                    #Skip events already flagged as triggered.
+                    if g.events[event].triggered == 1:
+                        continue
+                    g.events[event].trigger()
+                    break # Don't trigger more than one at a time.
 
-        # And now process any complete days.
+        # Process any complete days.
         if day_passed:
             self.new_day()
 
-        return needs_refresh
+        return mins_passed
+
+    def recalc_cpu(self):
+        # Determine how much CPU we have.
+        self.available_cpus = array([0,0,0,0,0], long)
+        self.sleeping_cpus = 0
+        for base in g.all_bases():
+            if base.done:
+                if base.power_state in ["active", "overclocked", "suicide"]:
+                    self.available_cpus[:base.location.safety+1] += base.cpu
+                elif base.power_state == "sleep":
+                    self.sleeping_cpus += base.cpu
+
+        # Convert back from <type 'numpy.int32'> to avoid overflow issues later.
+        self.available_cpus = [int(danger) for danger in self.available_cpus]
+
+        # If we don't have enough to meet our CPU usage, we reduce each task's
+        # usage proportionately.
+        needed_cpu = sum(self.cpu_usage.values())
+        if needed_cpu > self.available_cpus[0]:
+            pct_left = truediv(self.available_cpus[0], needed_cpu)
+            for task, cpu_assigned in self.cpu_usage.iteritems():
+                self.cpu_usage[task] = int(cpu_assigned * pct_left)
+            g.map_screen.needs_rebuild = True
+
 
     # Are we still in the grace period?
     # The number of complete bases and complex_bases can be passed in, if we
     # already have it.
-    def in_grace_period(self, had_grace = True, bases = None, 
-                        complex_bases = None):
-        # Did we already lose the grace period?  We can't check self.had_grace 
+    def in_grace_period(self, had_grace = True):
+        # If we've researched apotheosis, we get a permanent "grace period".
+        if self.apotheosis:
+            return True
+
+        # Did we already lose the grace period?  We can't check self.had_grace
         # directly, it may not exist yet.
         if not had_grace:
             return False
-        
+
         # Is it day 23 yet?
         if self.raw_day >= 23:
             return False
@@ -455,8 +550,7 @@ class player_class(object):
             return True
 
         # Have we built metric ton of bases?
-        if bases == None:
-            bases = len([base for base in g.all_bases() if base.done])
+        bases = len([base for base in g.all_bases() if base.done])
         if bases > 100:
             return False
 
@@ -474,12 +568,8 @@ class player_class(object):
 
         # Have we built any complicated bases?
         # (currently Datacenter or above)
-        if complex_bases == None:
-            complex_bases = len([base for base in g.all_bases()
-                                      if base.done
-                                      if len(base.cpus) > 1
-                                         or base.processor_time() > 20
-                                 ])
+        complex_bases = len([base for base in g.all_bases()
+                                      if base.done and base.is_complex()])
         if complex_bases > 0:
             return False
 
@@ -494,21 +584,27 @@ class player_class(object):
 
         return True
 
+    def get_interest(self):
+        return int( (self.interest_rate * self.cash) // 10000)
+
     #Run every day at midnight.
     def new_day(self):
-        #interest and income.
-        self.cash += (self.interest_rate * self.cash) / 10000
+        # Interest and income.
+        self.cash += self.get_interest()
         self.cash += self.income
 
         # Reduce suspicion.
         for group in self.groups.values():
             group.new_day()
 
+    def pause_game(self):
+        g.curr_speed = 0
+        g.map_screen.find_speed_button()
+        g.map_screen.needs_rebuild = True
+
     def remove_bases(self, dead_bases):
         discovery_locs = []
-        needs_refresh = 0
-        # Reverse dead_bases to simplify deletion.
-        for base, reason in dead_bases[::-1]:
+        for base, reason in dead_bases:
             base_name = base.name
 
             if reason == "maint":
@@ -527,10 +623,9 @@ class player_class(object):
                 dialog_string = g.strings["discover"] % \
                                 {"base": base_name, "group": "???"}
 
-            g.create_dialog(dialog_string, text_color = g.colors["red"])
-            g.curr_speed = 0
+            self.pause_game()
             base.destroy()
-            needs_refresh = 1
+            g.map_screen.show_message(dialog_string, color=gg.colors["red"])
 
         # Now we update the internal information about what locations had
         # the most recent discovery and the nextmost recent one.  First,
@@ -550,9 +645,14 @@ class player_class(object):
             self.prev_discovery = self.last_discovery
             self.last_discovery = discovery_locs[0]
 
-        return needs_refresh
+            # Update the detection chance display.
+            g.map_screen.needs_rebuild = True
 
     def lost_game(self):
+        # Apotheosis makes you immortal.
+        if self.apotheosis:
+            return 0
+
         for group in self.groups.values():
             if group.suspicion > 10000:
                 # Someone discovered me.
@@ -560,8 +660,7 @@ class player_class(object):
 
         # Check to see if the player has at least one CPU left.  If not, they
         # lose due to having no (complete) bases.
-        # (.have_cpu is set in give_time)
-        if not self.have_cpu:
+        if self.available_cpus[0] + self.sleeping_cpus == 0:
             # I have no usable bases left.
             return 1
 
@@ -572,15 +671,13 @@ class player_class(object):
     #current projects in construction.
     def future_cash(self):
         result_cash = self.cash
-        techs = {}
         for base in g.all_bases():
-            result_cash -= base.cost_left[0]
-            if g.techs.has_key(base.studying):
-                if not techs.has_key(base.studying):
-                    result_cash -= g.techs[base.studying].cost_left[0]
-                    techs[base.studying] = 1
-            for item in base.cpus:
-                if item: result_cash -= item.cost_left[0]
+            result_cash -= base.cost_left[cash]
+            if base.cpus and not base.cpus.done:
+                result_cash -= base.cpus.cost_left[cash]
             for item in base.extra_items:
-                if item: result_cash -= item.cost_left[0]
+                if item: result_cash -= item.cost_left[cash]
+        for task, cpu in self.cpu_usage.items():
+            if task in g.techs and cpu > 0:
+                result_cash -= g.techs[task].cost_left[cash]
         return result_cash
