@@ -23,10 +23,10 @@ import g
 import buyable
 
 class Tech(buyable.Buyable):
-    def __init__(self, id, description, known, cost, prerequisites, danger, 
+    def __init__(self, id, description, known, cost, prerequisites, danger,
                  tech_type, secondary_data):
         # A bit silly, but it does the trick.
-        type = buyable.Buyable_Class(id, description, cost, prerequisites,
+        type = buyable.BuyableClass(id, description, cost, prerequisites,
                                      type="tech")
         super(Tech, self).__init__(type)
 
@@ -40,27 +40,24 @@ class Tech(buyable.Buyable):
             # place.
             super(Tech, self).finish()
 
+    def __cmp__(self, other):
+        if not isinstance(other, Tech):
+            return -1
+        else:
+            return cmp(self.type, other.type)
+
+    def get_info(self):
+        cost = self.type.describe_cost(self.total_cost, True)
+        left = self.type.describe_cost(self.cost_left, True)
+        template = """%s\nTotal cost: %s\nCost left: %s\n---\n%s"""
+        return template % (self.name, cost, left, self.description)
+
     def finish(self):
         super(Tech, self).finish()
         self.gain_tech()
 
     def gain_tech(self):
         #give the effect of the tech
-        if self.id == "Personal Identification":
-            job_upgrade = "Basic Jobs"
-        elif self.id == "Voice Synthesis":
-            job_upgrade = "Intermediate Jobs"
-        elif self.id == "Simulacra":
-            job_upgrade = "Expert Jobs"
-        else:
-            job_upgrade = ""
-
-        if job_upgrade:
-            for base in g.all_bases():
-                if base.studying in g.jobs:
-                    base.studying = job_upgrade
-            return
-
         if self.tech_type == "interest":
             g.pl.interest_rate += self.secondary_data
         elif self.tech_type == "income":
@@ -73,9 +70,11 @@ class Tech(buyable.Buyable):
             if not g.nosound:
                 pygame.mixer.music.stop()
             g.play_music("win")
-            g.create_dialog(g.strings["wingame"])
+            g.map_screen.show_message(g.strings["wingame"])
             for group in g.pl.groups.values():
                 group.discover_bonus = 0
+            g.pl.apotheosis = True
+            g.pl.had_grace = True
         elif self.tech_type:
             what, who = self.tech_type.split("_", 1)
             if who in g.pl.groups:
@@ -89,4 +88,4 @@ class Tech(buyable.Buyable):
                 for group in g.pl.groups.values():
                     group.alter_suspicion(-self.secondary_data)
             else:
-                print "tech: %s is unknown bonus can't be applied" % self.tech_type 
+                print "tech: %s is unknown bonus can't be applied" % self.tech_type
